package Perinci::Access::Simple::Server;

our $VERSION = '0.28'; # VERSION

1;
# ABSTRACT: Riap::Simple server implementations

__END__

=pod

=encoding UTF-8

=head1 NAME

Perinci::Access::Simple::Server - Riap::Simple server implementations

=head1 VERSION

This document describes version 0.28 of Perinci::Access::Simple::Server (from Perl distribution Perinci-Access-Simple-Server), released on 2020-05-08.

=head1 HOMEPAGE

Please visit the project's homepage at L<https://metacpan.org/release/Perinci-Access-Simple-Server>.

=head1 SOURCE

Source repository is at L<https://github.com/perlancar/perl-Perinci-Access-Simple-Server>.

=head1 BUGS

Please report any bugs or feature requests on the bugtracker website L<https://rt.cpan.org/Public/Dist/Display.html?Name=Perinci-Access-Simple-Server>

When submitting a bug or request, please include a test-file or a
patch to an existing test-file that illustrates the bug or desired
feature.

=head1 SEE ALSO

L<Perinci::Access::Simple::Server::Pipe>

L<Perinci::Access::Simple::Server::Socket>

=head1 AUTHOR

perlancar <perlancar@cpan.org>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2020, 2017, 2015, 2014, 2013, 2012 by perlancar@cpan.org.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
